package basic;

import java.util.Vector;

public class BasicString {

  private Vector data;

  public BasicString() {
    data = new Vector();
  }

  public BasicString(String str) {
    this();
    str = str.toUpperCase();
    // convert str to char array
    char[] array = new char[str.length()];
    for(int i=0; i<array.length; i++) {
      array[i] = str.charAt(i);
    }
    setData(array);
  }
  public BasicString(char[] c) {
    this();
    setData(c);
  }
  public BasicString(int offset, int len, Vector s2) {
    this();
    char[] c2 = new char[len];
    for(int i=0; i<len; i++) {
      c2[i] = ((Character)s2.elementAt(offset+i)).charValue();
    }
    setData(c2);
  }

  public BasicString(double d) {
    this(String.valueOf(d));
  }

  public BasicString(char c) {
    this();

    char[] array = new char[1];
    array[0] = c;
    setData(array);
  }

  public boolean equals(Object o) {

    if (o == null) {
      return false;
    }

    boolean answer = false;

    if (o instanceof BasicString) {
      BasicString s2 = (BasicString)o;
      Vector v1 = this.getData();
      Vector v2 = s2.getData();

      if (v1.size() == v2.size()) {
        answer = true;
        for(int i=0; i<v1.size(); i++) {
          if (!v1.elementAt(i).equals(v2.elementAt(i))) {
            answer  =false;
            break;
          }
        }
      }
    }
    return answer;
  }

  public void append(BasicString s2) {

    if (s2 == null) {
      return;
    }

    Vector v1 = getData();
    Vector v2 = s2.getData();

    for(int i=0; i<v2.size();i++) {
      v1.addElement(v2.elementAt(i));
    }
  }
  public void append(char ch) {

    Vector data = getData();
    data.addElement(new Character(ch));
  }

  public char getCharAt(int index) {

    Character ch = (Character)data.elementAt(index);

    return ch.charValue();
  }

  public Vector getData() {
    return this.data;
  }
  public void setData(char[] c) {

    this.data.removeAllElements();
    for(int i=0; i<c.length; i++) {
      this.data.addElement(new Character(c[i]));
    }
  }
  public void setData(Vector v) {
    this.data = v;
  }

  public int compareTo(BasicString s2) {

    Vector v1 = getData();
    Vector v2 = s2.getData();

    int len1 = v1.size();
    int len2 = v2.size();
    int n = Math.min(len1, len2);
    int i = 0;
    int j = 0;

    while (n-- != 0) {
      char c1 = ((Character)v1.elementAt(i++)).charValue();
      char c2 = ((Character)v2.elementAt(j++)).charValue();
      if (c1 != c2) {
        return c1 - c2;
      }
    }
    return len1 - len2;
  }

  public char charAt(int index) {

    Character ch = (Character)data.elementAt(index);

    return ch.charValue();
  }

  public int length() {
    return data.size();
  }
  public BasicString trim() {
    int len = data.size();
    int st = 0;
    int off = 0;
    Vector v = getData();    /* avoid getfield opcode */

    char[] val = new char[v.size()];
    for(int i=0; i<val.length; i++) {
      val[i] = ((Character)v.elementAt(i)).charValue();
    }


    while ((st < len) && (val[off + st] <= ' ')) {
      st++;
    }
    while ((st < len) && (val[off + len - 1] <= ' ')) {
      len--;
    }

    if (st > 0 || len<data.size()) {

      BasicString trimmed = null;
      try {
        trimmed = substring(st, len);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
      return trimmed;
    } else {
      return this;
    }
  }

  public BasicString substring(int beginIndex) throws StringIndexOutOfBoundsException {
    return substring(beginIndex, length());
  }

  public BasicString substring(int beginIndex, int endIndex) throws StringIndexOutOfBoundsException {
    if (beginIndex < 0) {
      throw new StringIndexOutOfBoundsException(beginIndex);
    }
    if (endIndex > data.size()) {
      throw new StringIndexOutOfBoundsException(endIndex);
    }
    if (beginIndex > endIndex) {
      throw new StringIndexOutOfBoundsException(endIndex - beginIndex);
    }
    return ((beginIndex == 0) && (endIndex == data.size())) ? this :
    new BasicString(0 + beginIndex, endIndex - beginIndex, data);
  }

  public String getAsciiString() {

    char[] c = new char[data.size()];
    for(int i=0; i<c.length; i++) {
      c[i] = ((Character)data.elementAt(i)).charValue();
    }
    return new String(c);
  }
  public String toString() {
    return getAsciiString();
  }
}

