/*
 * ConstantExpression.java - An expression which is simply a constant.
 *
 * Copyright (c) 1996 Chuck McManis, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * CHUCK MCMANIS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. CHUCK MCMANIS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
 * OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package basic;
import java.io.PrintStream;

import jp.gr.java_conf.sol.basic.*;

/**
 * This class implements the simplest possible expression, a constant.
 */
class ConstantExpression extends Expression {
    private double v;
    private BasicString strCons;

    ConstantExpression(double a) {
        super();
        v = a;
    }

    ConstantExpression(BasicString a) {
        super();
        strCons = a;
    }

    void print(PrintStream p) {

      BasicString result = null;

      if (strCons == null) {
        result = new BasicString(v);
      } else {
        result = strCons;
      }

      p.print(result);
    }

    double value(Program pgm) throws BASICRuntimeError {
        if (strCons != null)
            return 0;
        return v;
    }

    String unparse() {
        if (strCons != null) {
            return ("\""+strCons+"\"");
        }
        return ""+v;
    }

    BasicString stringValue(Program pgm, int c) throws BASICRuntimeError {
        if (strCons != null)
            return strCons;
        return new BasicString(v);
    }

    BasicString stringValue(Program pgm) throws BASICRuntimeError {
        if (strCons != null)
            return strCons;
        return new BasicString(v);
    }

    boolean isString() {
        return (strCons != null);
    }

    public String toString() {
        if (strCons != null)
            return strCons.getAsciiString();
        return ""+v;
    }
}
