package jp.gr.java_conf.sol.basic;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;
import java.net.*;
import basic.*;

public class ConsoleDefault extends Canvas implements Console, Runnable, KeyListener {

  //screen
  private int screenWidth;
  private int screenHeight;
  private int fontWidth;
  private int fontHeight;
  protected   CustomFont    c_font;
  protected   ScreenBuffer  screenBuffer;
  protected   Image         offscreen;

  //cursor
  private     Point cursorPoint;
  protected   boolean cursorFlag;
  protected   Thread blinkThread;

  // key input
  protected   Vector keyBuffer = new Vector();
  protected   InputStream inputStream;
  protected   CommandInterpreter interpreter;

  public ConsoleDefault() {

    super();

    // set screen
    setScreenWidth(40);
    setScreenHeight(25);

    addKeyListener(this);
    
    // cursor
    cursorFlag = false;
    setCursorPoint(new Point(0,0));
  }

  public void finalize() {

    offscreen.flush();
    blinkThread.stop();

    offscreen = null;
    blinkThread = null;
  }

  protected void initScreen() {

    screenBuffer = new ScreenBuffer(getScreenWidth(), getScreenHeight(), 0x00);
    offscreen = this.createImage(getScreenWidth()*getFontWidth(), getScreenHeight()*getFontHeight());
  }

  public void setInterpreter(CommandInterpreter interpreter) {
    this.interpreter = interpreter;
  }

  public InputStream getInputStream() {

    if (this.inputStream == null) {
      setInputStream(new ConsoleInputStream());
    }

    return this.inputStream;
  }

  public OutputStream getOutputStream() {
    return new ConsoleOutputStream();
  }

  public void setInputStream(InputStream is) {
    this.inputStream = is;
  }

  protected  void setKeyBuffer(Vector buffer) {

    if (buffer != null) {
      for(int i=0; i<buffer.size(); i++) {
        keyBuffer.addElement(buffer.elementAt(i));
      }
    }
    synchronized(inputStream) {
      inputStream.notify();
    }
  }

  protected Vector getKeyBuffer() {
    return keyBuffer;
  }

  public void run() {

    while(true) { // cursor bliking never ends.
      blink();
      try {
        blinkThread.sleep(500);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  protected void blink() {

    cursorFlag = cursorFlag ^ true;
    ///*
    // cursor
    Point p = getCursorPoint();
    if (cursorFlag) {
      drawCursor(p);
    } else {
      eraseCursor(p);
    }
  }

  public void paint(Graphics g) {

    if (c_font == null) {
      return; // not initialized
    }

    if (offscreen == null) {
      offscreen = this.createImage(getScreenWidth()*getFontWidth(), getScreenHeight()*getFontHeight());
    }

    // copy offscreen to screen
    g.drawImage(offscreen, 0,0, null);
  }

  public void repaintLine(Integer fromLine, Integer toLine) {

    int from = fromLine.intValue();
    int to = toLine.intValue();

    Graphics g = getGraphics();
    Graphics g2= offscreen.getGraphics();
    for(int y=from; y<=to; y++) {
      for(int x=0; x<getScreenWidth(); x++) {
        CustomCharacter value = screenBuffer.get(x,y);

        // to offscreen
        c_font.drawImage(value, new Point(x, y), g2);
        // to screen
        c_font.drawImage(value, new Point(x, y), g);
      }
    }
    g.dispose();
    g2.dispose();
  }

  public void drawCursor(Point p) {

    Graphics g = getGraphics();

    c_font.drawImageIndex(0xef, 0xffffffff, 0xff000000, p, g);

    g.dispose();
  }

  public void eraseCursor(Point p) {

    Graphics g = getGraphics();

    int sx1 = p.x*getFontWidth();
    int sy1 = p.y*getFontHeight();

    if (c_font != null) {
      CustomCharacter ch = screenBuffer.get(p.x, p.y);
      if (ch != null) {
        c_font.drawImage(ch, p, g);
      }
    }
    g.dispose();
  }

  public Dimension getPreferredSize() {

    Dimension dim = new Dimension(getFontWidth()*getScreenWidth(), getFontHeight()*getScreenHeight());

    return dim;
  }

  public void setScreenWidth(int newScreenWidth) {
    screenWidth = newScreenWidth;
  }

  public void setScreenHeight(int newScreenHeight) {
    screenHeight = newScreenHeight;
  }

  public int getScreenWidth() {
    return screenWidth;
  }

  public int getScreenHeight() {
    return screenHeight;
  }

  public BasicString getStartupMsg() {

    String msg  =
      "------------------------------------------\n"+
      "default basic\n"+
      "------------------------------------------\n";

    return new BasicString(msg);

  }

  public void printStartupMsg() {
    BasicString msg = getStartupMsg();
    for(int i=0; i<msg.length(); i++) {
      print(msg.charAt(i));
    }
  }

  public void setCursorPoint(Point newCursorPoint) {
    cursorPoint = newCursorPoint;
  }

  public Point getCursorPoint() {
    return cursorPoint;
  }

  protected void repaintOffscreen() {

    // copy screenBuffer to offscreen

    Color bg = getBackground();
    int w = getFontWidth();
    int h= getFontHeight();
    int s_w = getScreenWidth();
    int s_h = getScreenHeight();
    Image img = null;
    CustomCharacter ch = null;
    
    Graphics g2 = offscreen.getGraphics();
    for(int y=0; y<s_h; y++) {
      for(int x=0; x<s_w; x++) {
        ch = screenBuffer.get(x, y);
        c_font.drawImage(ch, new Point(x, y), g2);
      }
    }
    g2.dispose();
  }

  protected  void printChar2Screen(Point p, CustomCharacter c) {
    /*
    eraseCursor(p);

    // to buffer
    screenBuffer.set(c, p.x, p.y);

    // to offscreen
    Image img = font.getImage(c);
    Graphics g = offscreen.getGraphics();
    g.drawImage(img, p.x*getFontWidth(), p.y*getFontHeight(), getFontWidth(),getFontHeight(), Color.white, null);
    g.dispose();

    // to window
    Graphics g2 = getGraphics();
    synchronized(g2) {
      g2.drawImage(img, p.x*getFontWidth(), p.y*getFontHeight(), getFontWidth(),getFontHeight(), Color.white, null);
    }
    g2.dispose();
    */
  }

  public CustomCharacter get(Point p) {
    return screenBuffer.get(p.x, p.y);
  }


  public void print(CustomCharacter c) {

    int code = c.getCode();

    if (code >=0x20) {
      Point p = getCursorPoint();
      printChar2Screen(p, c);
      if (++p.x>=getScreenWidth()) {
        screenBuffer.connectLine(p.y);
        doLF();
      }
    } else {
      switch (code) {

        case 0x08:  // tab
        print ("        ");
        break;

        case 0x0A:  //
        doLF();
        break;

        case 0x15://  home code
        setCursorPoint(new Point(0,0));
        break;

        case 0x16://  cls code
        screenBuffer.clear();
        repaintOffscreen();
        repaint();
        setCursorPoint(new Point(0,0));
        break;
      }
    }
  }

  public void print(byte[] b) {
    for(int i=0; i<b.length; i++) {
      print(screenBuffer.createCharacter(b[i]));
    }
  }

  public void print(char c) {
    print(screenBuffer.createCharacter(c));
  }

  public void print(String str) {
    for(int i=0; i<str.length(); i++) {
      print(str.charAt(i));
    }
  }

  public void println(String str) {
    print(str);
    doLF();
  }

  public void doLF() {

    Point p = getCursorPoint();
    eraseCursor(p);

    p.x = 0;
    p.y++;

    if (p.y>=getScreenHeight()) {
      p.y = getScreenHeight()-1;
      doScroll();
    }
  }

  public void doScroll() {

    screenBuffer.scroll();
    
    //repaintOffscreen();
    Graphics g2 = offscreen.getGraphics();
    g2.drawImage(offscreen, 0,0, getScreenWidth()*getFontWidth(), (getScreenHeight()-1)*getFontHeight(),
                            0, getFontHeight(), getScreenWidth()*getFontWidth(), getScreenHeight()*getFontHeight(), null);

    g2.setColor(this.getBackground());
    g2.fillRect(0,(getScreenHeight()-1)*getFontHeight(), getScreenWidth()*getFontWidth(), getScreenHeight()*getFontHeight());
    g2.dispose();

    // copy offscreen to screen
    Graphics g = this.getGraphics();

    g.drawImage(offscreen, 0,0, null);
    g.dispose();
  }

  public void setFontWidth(int newFontWidth) {
    fontWidth = newFontWidth;
  }

  public int getFontWidth() {
    return fontWidth;
  }

  public void setFontHeight(int newFontHeight) {
    fontHeight = newFontHeight;
  }

  public int getFontHeight() {
    return fontHeight;
  }

  public void keyTyped(KeyEvent e) {
  }
  public void keyPressed(KeyEvent e) {
  }
  public void keyReleased(KeyEvent e) {
  }

  class ConsoleInputStream extends InputStream {
    ConsoleInputStream() {
    }

    public int available() throws IOException {
      return getKeyBuffer().size();
    }

    public int read() throws IOException {

      int answer = -1;

      while (keyBuffer.size() == 0) {
        synchronized (this) {
          try {

            this.wait();

          } catch (Exception ex) {
            ex.printStackTrace();
          }
        }
      }

      synchronized (keyBuffer) {
        answer = ((CustomCharacter)keyBuffer.elementAt(0)).getCode();
        keyBuffer.removeElementAt(0);
      }


      return answer;
    }
  }

  class ConsoleOutputStream extends OutputStream {
    ConsoleOutputStream() {
    }

    public void write(byte[] b) throws java.io.IOException {
      print(b);
    }
    public void write(int b) throws IOException {
      print((char)b);
    }
  }

  public boolean isFocusTraversable() {
    return true;
  }

}
