package jp.gr.java_conf.sol.basic;

import java.util.*;
import java.awt.*;

public class ScreenBuffer {

  private Vector buffer;
  private int width;
  private int height;
  private int initialValue;

  public ScreenBuffer(int width, int height, int initialValue) {
    buffer = new Vector();

    this.width = width;
    this.height = height;
    this.initialValue = initialValue;

    for(int i=0; i<height; i++) {
      buffer.addElement(createEmptyLine());
    }
  }

  public void clear() {
    for(int y =0; y<this.height; y++) {
      for(int x=0; x<this.width; x++) {
        set(createCharacter(0), x, y);
      }
    }
  }

  protected Line createEmptyLine() {

    Line line = new Line();
    for(int i=0; i<width; i++) {
      line.addElement(createCharacter(initialValue));
    }

    return line;
  }

  public CustomCharacter createCharacter(int value) {
    return new CustomCharacter(value);
  }

  public synchronized void set(CustomCharacter c, int x, int y) {
    Line line = (Line)buffer.elementAt(y);
    CustomCharacter obj = (CustomCharacter)line.elementAt(x);

    line.setElementAt(c, x);
  }

  public synchronized CustomCharacter get(int x, int y) {

    CustomCharacter obj = null;
    if (x>=0 && x<width && y>=0 && y<height) {
      Line line = (Line)buffer.elementAt(y);
      obj = (CustomCharacter)line.elementAt(x);
    }

    return obj;
  }

  public Vector getLine(int y) {
    Line line = (Line)buffer.elementAt(y);

    Vector v = new Vector();
    int x =0;

    boolean finish = false;
    while(!finish) {
      CustomCharacter ch = get(x++, y);
      if (ch.getCode() != initialValue) {
        v.addElement(ch);
      } else {
        finish = true;
      }
    }

    return v;
  }

  /**

  @return cursor point after delete character
  */
  public Point delete(int x, int y, Ref fromLine, Ref toLine) {

    // set lines to be repaint
    if (fromLine.getObject() == null) {
      fromLine.setObject(new Integer(y));
    }
    toLine.setObject(new Integer(y));

    Point p = new Point(x,y);

    Line line = (Line)buffer.elementAt(y);

    if (x>0) {
      // remove prev char
      line.removeElementAt(x-1);
      p.x--;
    } else { // x == 0
      if (y>0) {
        Line pLine = (Line)buffer.elementAt(y-1);
        if (pLine.isNextLine()) { // if connected
          // remove last char of previous line

          CustomCharacter head = (CustomCharacter)line.elementAt(0);
          line.removeElementAt(0); // remove head

          pLine.removeElementAt(width-1);
          pLine.addElement(head);
          p.x = width-1;
          p.y-=1;
        } else {
          // remove this char.
          line.removeElementAt(0);
        }
      } else {  //x==0,y==0
        // remove this char
        line.removeElementAt(0);
      }
    }

    // and feed
    if (line.isNextLine()) {
      CustomCharacter head = get(0,y+1);
      line.addElement(head);
      set(head, width-1, y);
      delete(1,y+1, fromLine, toLine);
    } else {
      line.addElement(createCharacter(initialValue));
    }

    buffer.setElementAt(line, y);

    return p;
  }

  /** insert character to buffer
  @param c charactre
  @param x point;
  @param y point
  @param fromLine start line to be repaint after insert
  @param toline end line to be repaint after insert
  */
  public void insert(CustomCharacter c, int x, int y, Ref fromLine, Ref toLine) {

    if (fromLine.getObject() == null) {
      fromLine.setObject(new Integer(y));
    }
    toLine.setObject(new Integer(y));

    Line line = (Line)buffer.elementAt(y);

    // get last line
    boolean finish = false;
    int tempY = y;
    Line lastLine = line;
    while(!finish) {
      if (lastLine.isNextLine()) {
        lastLine = (Line)buffer.elementAt(++tempY);
      } else {
        finish = true;
      }
    }

    // get last char
    CustomCharacter last = (CustomCharacter)lastLine.elementAt(width-1);
    if (last.getCode() == initialValue) {  // can insert
      line.insertElementAt(c, x);

      if (line.isNextLine()) {
        // remove last char and insert it to head of next line
        CustomCharacter lastCharAtLine = (CustomCharacter)line.elementAt(width);
        line.removeElementAt(width);

        insert(lastCharAtLine, 0, y+1, fromLine, toLine);
      }
    } else {  // last line is filled with character, can not insert anything.
    }
  }

  public void scroll() {
    synchronized(buffer) {
      buffer.removeElementAt(0);
      buffer.addElement(createEmptyLine());
    }
  }

  public void connectLine(int y) {
    Line line = (Line)buffer.elementAt(y);
    line.setNextLine(true);
  }

  class Line extends Vector {

    private boolean nextLine;

    Line() {
      super();
      nextLine = false;
    }

    boolean isNextLine() {
      return nextLine;
    }
    void setNextLine(boolean f) {
      this.nextLine = f;
    }
  }
}

