package jp.gr.java_conf.sol.basic.mz700;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import jp.gr.java_conf.sol.basic.*;
import basic.*;

public class FontMZ700 implements CustomFont {

  Image masterImg;
  Component parent;

  int fontWidth;
  int fontHeight;

  public FontMZ700(Component parent, int fontWidth, int fontHeight, InputStream fontInput, DialogLoadingFont dlog) throws IOException {
    this.parent = parent;
    this.fontWidth = fontWidth;
    this.fontHeight = fontHeight;
    loadFont(fontWidth, fontHeight, fontInput, dlog);
  }

  public void finalize() {
    if (masterImg != null) {
      masterImg.flush();
      masterImg = null;
    }
  }

  public void drawImage(CustomCharacter ch, Point p, Graphics g) {

    MZ700Character mzch = (MZ700Character)ch;

    int code = mzch.getCode();
    int page = mzch.getPage();
    int index = table[code][1]+page*0x100;
    int fg = mzch.getForeColorIndex();
    int bg = mzch.getBackColorIndex();

    drawImageIndex(index, fg, bg, p, g);
  }

  public void drawImageIndex(int index, int fg, int bg, Point p, Graphics g) {
    int sx = index*fontWidth;
    int sy = bg*fontHeight*8+fg*fontHeight;

    int dx = p.x*fontWidth;
    int dy = p.y*fontHeight;

    g.drawImage(masterImg, dx, dy, dx+fontWidth, dy+fontWidth, sx, sy, sx+fontWidth, sy+fontHeight, null);
  }
  /*
  private void loadFont(int fontWidth, int fontHeight, InputStream fontStream, DialogLoadingFont dlog) throws IOException {

  if (masterImg != null) {
  masterImg.flush();
  masterImg = null;
  }

  System.gc();

  int bitWidth = fontWidth/8;
  int bitHeight= fontHeight/8;

  // create masterImg
  masterImg = parent.createImage(0x200*fontWidth, 64*fontHeight);

  // create fontImage
  Image fontImage = parent.createImage(0x200*fontWidth, fontHeight);

  Graphics g = fontImage.getGraphics();

  // create master image(fg=7,bg=0)
  int sy = 0*fontHeight*8+7*fontHeight;
  int sx;
  char bit;
  BufferedReader reader = new BufferedReader(new InputStreamReader(fontStream));
  for(int i=0; i<0x200; i++) {
  String line = reader.readLine(); //#0x000
  for(int y=0; y<8; y++) {
  line = reader.readLine();
  for(int x=0; x<8; x++) {
  bit = line.charAt(x);
  sx = i*fontWidth;
  if (bit == 'o') {
  g.setColor(Color.white);
  } else {
  g.setColor(Color.black);
  }
  g.fillRect(i*fontWidth+x*bitWidth, y*bitHeight, bitWidth, bitHeight);
  }
  }
  reader.readLine();  // empty line
  }
  reader.close();
  g.dispose();

  // copy font image 2 masterImg widh filtering
  g = masterImg.getGraphics();
  MZColorFilter filter = new MZColorFilter();
  ImageProducer producer = new FilteredImageSource(masterImg.getSource(), filter);
  for(int bg=0; bg<8; bg++) {
  for(int fg=0; fg<8; fg++) {
  dlog.setProgress( (bg*8+fg)*100/64+"%");
  filter.setFG(fg);
  filter.setBG(bg);
  Image image2 = parent.createImage(new FilteredImageSource(fontImage.getSource(), filter));

  int sy2 = bg*8*fontHeight+fg*fontHeight;

  g.drawImage(image2, 0, sy2, 0x200*fontWidth, sy2+fontHeight,
  0,0, 0x200*fontWidth, fontHeight, null);
  }
  }

  g.dispose();

  }

  class MZColorFilter extends RGBImageFilter {

  int fgRGB;
  int bgRGB;

  public MZColorFilter() {
  }

  public int filterRGB(int x, int y, int rgb) {

  if (rgb == 0xffffffff) {  // white
  return fgRGB;
  } else {
  return bgRGB;
  }
  }

  void setFG(int fg) {
  this.fgRGB = MZ700Character.getRGB(fg);
  }
  void setBG(int bg) {
  this.bgRGB = MZ700Character.getRGB(bg);
  }
  }*/

  private void loadFont(int fontWidth, int fontHeight, InputStream fontStream, DialogLoadingFont dlog) throws IOException {

    if (masterImg != null) {
      masterImg.flush();
      masterImg = null;
    }

    System.gc();

    int bitWidth = fontWidth/8;
    int bitHeight= fontHeight/8;

    // create masterImg
    masterImg = parent.createImage(0x200*fontWidth, 64*fontHeight);
    Graphics g = masterImg.getGraphics();

    BufferedReader reader = new BufferedReader(new InputStreamReader(fontStream));
    for(int i=0; i<0x200; i++) {

      dlog.setProgress(100*i/0x200+"%");

      String line = reader.readLine(); //#0x000
      int sx,sy;
      Color fColor, bColor;
      char bit;
      for(int y=0; y<8; y++) {
        line = reader.readLine();
        for(int x=0; x<8; x++) {
          bit = line.charAt(x);
          sx = i*fontWidth;
          for(int bg = 0; bg<8; bg++) {
            for(int fg = 0; fg<8; fg++) {
              if (bit == 'o') {
                g.setColor(MZ700Character.getColor(fg));
              } else {
                g.setColor(MZ700Character.getColor(bg));
              }
              sy = bg*fontHeight*8+fg*fontHeight;
              g.fillRect(sx+x*bitWidth, sy+y*bitHeight, bitWidth, bitHeight);
            }
          }
        }
      }
      reader.readLine();  // empty line
    }
    reader.close();
    g.dispose();
  }
  // (MZ700 character code - MZ700 font)
  static final int[][] table = {
    {0x00,0x000},
    {0x01,0x000},
    {0x02,0x000},
    {0x03,0x000},
    {0x04,0x000},
    {0x05,0x000},
    {0x06,0x000},
    {0x07,0x000},
    {0x08,0x000},
    {0x09,0x000},
    {0x0a,0x000},
    {0x0b,0x000},
    {0x0c,0x000},
    {0x0d,0x000},
    {0x0e,0x000},
    {0x0f,0x000},
    {0x10,0x000},
    {0x11,0x000},
    {0x12,0x000},
    {0x13,0x000},
    {0x14,0x000},
    {0x15,0x000},
    {0x16,0x000},
    {0x17,0x000},
    {0x18,0x000},
    {0x19,0x000},
    {0x1a,0x000},
    {0x1b,0x000},
    {0x1c,0x000},
    {0x1d,0x000},
    {0x1e,0x000},
    {0x1f,0x000},
    {0x20,0x000},// SPC
    {0x21,0x061},// !
    {0x22,0x062},// "
    {0x23,0x063},// #
    {0x24,0x064},// $
    {0x25,0x065},// %
    {0x26,0x066},// &
    {0x27,0x067},// '
    {0x28,0x068},// (
    {0x29,0x069},// )
    {0x2a,0x06b},// *
    {0x2b,0x06a},// +
    {0x2c,0x02f},// ,
    {0x2d,0x02a},// -
    {0x2e,0x02e},// .
    {0x2f,0x02d},// /
    {0x30,0x020},// 0
    {0x31,0x021},// 1
    {0x32,0x022},// 2
    {0x33,0x023},// 3
    {0x34,0x024},// 4
    {0x35,0x025},// 5
    {0x36,0x026},// 6
    {0x37,0x027},// 7
    {0x38,0x028},// 8
    {0x39,0x029},// 9
    {0x3a,0x04f},// :
    {0x3b,0x02c},// ;
    {0x3c,0x051},// <
    {0x3d,0x02B},// =
    {0x3e,0x057},// >
    {0x3f,0x049},// ?
    {0x40,0x055},// @
    {0x41,0x001},// A
    {0x42,0x002},// B
    {0x43,0x003},// C
    {0x44,0x004},// D
    {0x45,0x005},// E
    {0x46,0x006},// F
    {0x47,0x007},// G
    {0x48,0x008},// H
    {0x49,0x009},// I
    {0x4a,0x00a},// J
    {0x4b,0x00b},// K
    {0x4c,0x00c},// L
    {0x4d,0x00d},// M
    {0x4e,0x00e},// N
    {0x4f,0x00f},// O
    {0x50,0x010},// P
    {0x51,0x011},// Q
    {0x52,0x012},// R
    {0x53,0x013},// S
    {0x54,0x014},// T
    {0x55,0x015},// U
    {0x56,0x016},// V
    {0x57,0x017},// W
    {0x58,0x018},// X
    {0x59,0x019},// Y
    {0x5a,0x01a},// Z
    {0x5b,0x052},// [
    {0x5c,0x059},// \
    {0x5d,0x054},// ]
    {0x5e,0x050},// upper arrow
    {0x5f,0x045},// left arrow
    {0x60,0x0c7},// ufo
    {0x61,0x0c8},// right car?
    {0x62,0x0c9},// upper car?
    {0x63,0x0ca},// upper person
    {0x64,0x0cb},// left person
    {0x65,0x0cc},// left person
    {0x66,0x0cd},// lower person
    {0x67,0x0ce},// face filled
    {0x68,0x0cf},// face
    {0x69,0x0df},// snake
    {0x6a,0x0e7},// right arrow2
    {0x6b,0x0e8},// left arrow2
    {0x6c,0x0e9},// lower arrow2
    {0x6d,0x0ea},// lower srrow3
    {0x6e,0x0ec},//
    {0x6f,0x0ed},//
    {0x70,0x0d0},// Sunday
    {0x71,0x0d1},// Monday
    {0x72,0x0d2},//
    {0x73,0x0d3},//
    {0x74,0x0d4},//
    {0x75,0x0d5},//
    {0x76,0x0d6},//
    {0x77,0x0d7},//
    {0x78,0x0d8},//
    {0x79,0x0d9},//
    {0x7a,0x0da},//
    {0x7b,0x0db},//
    {0x7c,0x0dc},//
    {0x7d,0x0dd},//
    {0x7e,0x0de},//
    {0x7f,0x0c0},// load
    {0x80,0x080},// lower arrow
    {0x81,0x0bd},// maru
    {0x82,0x09d},// kakko
    {0x83,0x0b1},// kakko toji
    {0x84,0x0b5},// ten
    {0x85,0x0b9},// dot
    {0x86,0x0b4},// 
    {0x87,0x09e},// @
    {0x88,0x0b2},// B
    {0x89,0x0b6},// D
    {0x8a,0x0ba},// F
    {0x8b,0x0be},// H
    {0x8c,0x09f},// 
    {0x8d,0x0b3},// 
    {0x8e,0x0b7},// 
    {0x8f,0x0bb},// b
    {0x90,0x0bf},// -
    {0x91,0x0a3},// A
    {0x92,0x085},// C
    {0x93,0x0a4},// E
    {0x94,0x0a5},// G
    {0x95,0x0a6},// I
    {0x96,0x094},// J
    {0x97,0x087},// L
    {0x98,0x088},// N
    {0x99,0x09c},// P
    {0x9a,0x082},// R
    {0x9b,0x098},// T
    {0x9c,0x084},// V
    {0x9d,0x092},// X
    {0x9e,0x090},// Z
    {0x9f,0x083},// \
    {0xa0,0x091},// ^
    {0xa1,0x081},// `
    {0xa2,0x09a},// c
    {0xa3,0x097},// e
    {0xa4,0x093},// g
    {0xa5,0x095},// i
    {0xa6,0x089},// j
    {0xa7,0x0a1},// k
    {0xa8,0x0af},// l
    {0xa9,0x08b},// m
    {0xaa,0x086},// n
    {0xab,0x096},// q
    {0xac,0x0a2},// t
    {0xad,0x0ab},// w
    {0xae,0x0aa},// z
    {0xaf,0x08a},// }
    {0xb0,0x08e},// ~
    {0xb1,0x0b0},// 
    {0xb2,0x0ad},// 
    {0xb3,0x08d},// 
    {0xb4,0x0a7},// 
    {0xb5,0x0a8},// 
    {0xb6,0x0a9},// 
    {0xb7,0x08f},// 
    {0xb8,0x08c},// 
    {0xb9,0x0ae},// 
    {0xba,0x0ac},// 
    {0xbb,0x09b},// 
    {0xbc,0x0a0},// 
    {0xbd,0x099},// 
    {0xbe,0x0bc},// _
    {0xbf,0x0b8},// ܂
    {0xc0,0x040},// right arrow
    {0xc1,0x13b},// ?
    {0xc2,0x03a},// ?
    {0xc3,0x070},// ?
    {0xc4,0x03c},// ?
    {0xc5,0x071},// ?
    {0xc6,0x05a},// ?
    {0xc7,0x03d},// ?
    {0xc8,0x043},// ?
    {0xc9,0x056},// ?
    {0xca,0x03f},// ?
    {0xcb,0x01e},// ?
    {0xcc,0x04a},// ?
    {0xcd,0x01c},// ?
    {0xce,0x05d},// ?
    {0xcf,0x03e},// ?
    {0xd0,0x05c},// ?
    {0xd1,0x01f},// ?
    {0xd2,0x05f},// ?
    {0xd3,0x05e},// ?
    {0xd4,0x037},// ?
    {0xd5,0x03b},// ?
    {0xd6,0x07f},// ?
    {0xd7,0x036},// ?
    {0xd8,0x07a},// ?
    {0xd9,0x07e},// ?
    {0xda,0x033},// ?
    {0xdb,0x04b},// ?
    {0xdc,0x04c},// ?
    {0xdd,0x01d},// ?
    {0xde,0x06c},// ?
    {0xdf,0x05b},// ?
    {0xe0,0x078},// ?
    {0xe1,0x041},// ?
    {0xe2,0x035},// ?
    {0xe3,0x034},// ?
    {0xe4,0x074},// ?
    {0xe5,0x030},// ?
    {0xe6,0x038},// ?
    {0xe7,0x075},// ?
    {0xe8,0x039},// ?
    {0xe9,0x04d},// ?
    {0xea,0x06f},// ?
    {0xeb,0x06e},// ?
    {0xec,0x032},// ?
    {0xed,0x077},// ?
    {0xee,0x076},// ?
    {0xef,0x072},// ?
    {0xf0,0x073},// ?
    {0xf1,0x047},// ?
    {0xf2,0x07c},// ?
    {0xf3,0x053},// ?
    {0xf4,0x031},// ?
    {0xf5,0x04e},// ?
    {0xf6,0x06d},// ?
    {0xf7,0x048},// 
    {0xf8,0x046},// ?
    {0xf9,0x07d},// ?
    {0xfa,0x044},// ?
    {0xfb,0x01b},// ?
    {0xfc,0x058},// ?
    {0xfd,0x079},// ?
    {0xfe,0x042},// ?
    {0xff,0x060},// ?
  };
}
